@@ -10,19 +10,18 @@ import (
 	cohelpers "github.com/openshift/library-go/pkg/config/clusteroperator/v1helpers"
 	operatorhelpers "github.com/openshift/library-go/pkg/operator/v1helpers"
 	olm "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	mktconfig "github.com/operator-framework/operator-marketplace/pkg/apis/config/v1"
 	"github.com/operator-framework/operator-marketplace/pkg/apis/operators/v1"
+	"github.com/operator-framework/operator-marketplace/pkg/apis/operators/v2"
 	log "github.com/sirupsen/logrus"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/client-go/discovery"
-	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 )
 
 const (
-	// clusterOperatorName is the name of the cluster operator
+	// clusterOperatorName is the name of the ClusterOperator
 	clusterOperatorName = "marketplace"
 
 	// minSyncsBeforeReporting is the minimum number of syncs we wish to see
@@ -44,8 +43,12 @@ const (
 	syncsBeforeTruncate = 10000
 	syncTruncateValue   = 100
 
-	// coStatusReportInterval is the interval at which the cluster operator status is updated
+	// coStatusReportInterval is the interval at which the ClusterOperator status is updated
 	coStatusReportInterval = 20 * time.Second
+
+	// Marketplace is always upgradeable and should include this message in the Upgradeable
+	// ClusterOperatorStatus condition.
+	upgradeableMessage = "Marketplace is upgradeable"
 )
 
 // status will be a singleton
@@ -91,7 +94,7 @@ func StartReporting(cfg *rest.Config, mgr manager.Manager, namespace string, ver
 	// ensure instance is only created once.
 	once.Do(func() {
 		instance = new(cfg, mgr, namespace, version, stopCh)
-		// exit if cluster operator api is not present
+		// exit if ClusterOperator api is not present
 		if instance.coAPINotPresent {
 			return
 		}
@@ -108,25 +111,12 @@ func StartReporting(cfg *rest.Config, mgr manager.Manager, namespace string, ver
 
 // New returns an initialized status
 func new(cfg *rest.Config, mgr manager.Manager, namespace string, version string, stopCh <-chan struct{}) *status {
-	// Check if the ClusterOperator API is present on the cluster. If the API
-	// is not present on the cluster set the status.coAPINotPresent to true so
-	// status is not reported
-	k8sInterface, err := kubernetes.NewForConfig(cfg)
-	if err != nil {
-		log.Fatal("[status] " + err.Error())
-	}
-	err = discovery.ServerSupportsVersion(k8sInterface.Discovery(), schema.GroupVersion{
-		Group:   "config.openshift.io",
-		Version: "v1",
-	})
-
 	monitorDoneCh := make(chan struct{})
-	coAPINotPresent := false
-	if err != nil {
-		log.Warningf("[status] ClusterOperator API not present: %v", err)
-		coAPINotPresent = true
+	coAPINotPresent := !mktconfig.IsAPIAvailable()
+	if coAPINotPresent {
+		log.Warningf("[status] ClusterOperator API not present")
 		// If the co is not present, close the monitorDoneCh channel to prevent
-		// recievers from stalling
+		// receivers from stalling
 		close(monitorDoneCh)
 	}
 
@@ -141,6 +131,11 @@ func new(cfg *rest.Config, mgr manager.Manager, namespace string, version string
 		log.Fatal("[status] " + err.Error())
 	}
 
+	// If version is an empty string, warn that the operator is not a part of the OpenShift release payload.
+	if version == "" {
+		version = "OpenShift Independent Version"
+	}
+
 	return &status{
 		configClient:    configClient,
 		coAPINotPresent: coAPINotPresent,
@@ -247,6 +242,12 @@ func (s *status) updateStatus(previousStatus *configv1.ClusterOperatorStatus) er
 			}
 		}
 
+		// Log Conditions
+		log.Infof("[status] Attempting to set the ClusterOperator status conditions to:")
+		for _, statusCondition := range s.clusterOperator.Status.Conditions {
+			log.Infof("[status] ConditionType: %v ConditionStatus: %v ConditionMessage: %v", statusCondition.Type, statusCondition.Status, statusCondition.Message)
+		}
+
 		// Always update RelatedObjects to account for the upgrade case.
 		s.setRelatedObjects()
 
@@ -254,11 +255,7 @@ func (s *status) updateStatus(previousStatus *configv1.ClusterOperatorStatus) er
 		if err != nil {
 			return fmt.Errorf("Error %v updating ClusterOperator", err)
 		}
-		// Log Conditions
-		log.Infof("[status] Current ClusterOperator conditions:")
-		for _, statusCondition := range s.clusterOperator.Status.Conditions {
-			log.Infof("[status] ConditionType: %v\tConditionStatus: %v\tConditionMessage: %v", statusCondition.Type, statusCondition.Status, statusCondition.Message)
-		}
+		log.Info("[status] ClusterOperator status conditions updated.")
 	}
 	return err
 }
@@ -280,8 +277,8 @@ func (s *status) setRelatedObjects() {
 			Namespace: s.namespace,
 		},
 		{
-			Group:     v1.SchemeGroupVersion.Group,
-			Resource:  v1.CatalogSourceConfigKind,
+			Group:     v2.SchemeGroupVersion.Group,
+			Resource:  v2.CatalogSourceConfigKind,
 			Namespace: s.namespace,
 		},
 		{
@@ -313,7 +310,7 @@ func (s *status) syncChannelReceiver() {
 	}
 }
 
-// monitorClusterStatus updates the cluster operator's status based on
+// monitorClusterStatus updates the ClusterOperator's status based on
 // the number of successful syncs / total syncs
 func (s *status) monitorClusterStatus() {
 	// Signal to the main channel that we have stopped reporting status.
@@ -329,13 +326,14 @@ func (s *status) monitorClusterStatus() {
 			conditionListBuilder := clusterStatusListBuilder()
 			conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, msg, reason)
 			conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, reason)
+			conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
 			statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, reason)
 			statusErr := s.setStatus(statusConditions)
 			if statusErr != nil {
 				log.Error("[status] " + statusErr.Error())
 			}
 			return
-		// Attempt to update the cluster operator status whenever the seconds
+		// Attempt to update the ClusterOperator status whenever the seconds
 		// number of seconds defined by coStatusReportInterval passes.
 		case <-time.After(coStatusReportInterval):
 			// Log any status update errors after exit.
@@ -346,12 +344,13 @@ func (s *status) monitorClusterStatus() {
 				}
 			}()
 
-			// Create the cluster operator in the progressing state if it does not exist
+			// Create the ClusterOperator in the progressing state if it does not exist
 			// or if it is the first report.
 			if s.clusterOperator == nil {
 				reason := "OperatorStarting"
 				conditionListBuilder := clusterStatusListBuilder()
 				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Progressing towards release version: %s", s.version), reason)
+				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
 				msg := fmt.Sprintf("Determining status")
 				conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, reason)
 				statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, reason)
@@ -371,6 +370,7 @@ func (s *status) monitorClusterStatus() {
 				reason := "OperatorAvailable"
 				conditionListBuilder := clusterStatusListBuilder()
 				conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionFalse, fmt.Sprintf("Successfully progressed to release version: %s", s.version), reason)
+				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, reason)
 				statusConditions := conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionTrue, fmt.Sprintf("Available release version: %s", s.version), reason)
 				statusErr = s.setStatus(statusConditions)
 				break
@@ -381,6 +381,7 @@ func (s *status) monitorClusterStatus() {
 			if ratio != nil {
 				var statusConditions []configv1.ClusterOperatorStatusCondition
 				conditionListBuilder := clusterStatusListBuilder()
+				conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionTrue, upgradeableMessage, "OperatorAvailable")
 				if isSucceeding {
 					statusConditions = conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, fmt.Sprintf("Current CR sync ratio (%g) meets the expected success ratio (%g)", *ratio, successRatio), "OperandTransitionsSucceeding")
 				} else {
@@ -392,3 +393,16 @@ func (s *status) monitorClusterStatus() {
 		}
 	}
 }
+
+// ReportMigration sets the clusterOperator status to signal that migration logic is in progress,
+// while upgrading from openshift 4.1.z to openshift 4.2.z
+func ReportMigration(cfg *rest.Config, mgr manager.Manager, namespace string, version string, stopCh <-chan struct{}) error {
+	instance = new(cfg, mgr, namespace, version, stopCh)
+	conditionListBuilder := clusterStatusListBuilder()
+	conditionListBuilder(configv1.OperatorProgressing, configv1.ConditionTrue, fmt.Sprintf("Performing migration logic to progress towards release version: %s", instance.version), "Upgrading")
+	msg := fmt.Sprintf("Determining status")
+	conditionListBuilder(configv1.OperatorAvailable, configv1.ConditionFalse, msg, "Upgrading")
+	conditionListBuilder(configv1.OperatorUpgradeable, configv1.ConditionFalse, msg, "Upgrading")
+	statusConditions := conditionListBuilder(configv1.OperatorDegraded, configv1.ConditionFalse, msg, "Upgrading")
+	return instance.setStatus(statusConditions)
+}
